package net.wanji.opt.servicev2.judgeanalysis.impl;

import com.github.pagehelper.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.entity.GreenwaveInfoPO;
import net.wanji.databus.dao.mapper.GreenwaveInfoMapper;
import net.wanji.opt.common.TimeIntervalMerger;
import net.wanji.opt.common.enums.ConsgestionStatusEnum;
import net.wanji.opt.common.enums.GreenBeltDirEnum;
import net.wanji.opt.common.enums.RoadLevelEnum;
import net.wanji.opt.common.enums.WeekEnum;
import net.wanji.opt.dao.mapper.judgeanalysis.AnalysisGreenCongestionPeriodMapper;
import net.wanji.opt.dao.mapper.report.GreenWaveCrossRidMapper;
import net.wanji.opt.entity.judgeanalysis.AnalysisGreenCongestionPeriod;
import net.wanji.opt.entity.judgeanalysis.laneDataHist;
import net.wanji.opt.entity.report.GreenWaveCrossRidInfo;
import net.wanji.opt.servicev2.judgeanalysis.AnalysisGreenCongestionPeriodService;
import net.wanji.opt.vo2.report.AnalysisGreenCongestionPeriodVO;
import net.wanji.opt.vo2.report.GreenReportProblemDetailAnalysisResult;
import net.wanji.opt.vo2.report.GreenReportProblemDetailVO;
import net.wanji.opt.vo2.report.GreenReportProblemOverallAnalysisResult;
import net.wanji.opt.vo2.report.GreenReportProblemOverallDataVO;
import net.wanji.opt.vo2.report.GreenWaveDirProblemDataVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.temporal.TemporalAdjusters;
import java.time.temporal.WeekFields;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 * 干线拥堵时段实现类
 * </p>
 *
 * @author hwm
 * @since 2025-03-20
 */
@Service
@Slf4j
public class AnalysisGreenCongestionPeriodServiceImpl implements AnalysisGreenCongestionPeriodService {
    @Resource
    private AnalysisGreenCongestionPeriodMapper analysisGreenCongestionPeriodMapper;
    @Autowired
    private GreenWaveCrossRidMapper greenWaveCrossRidMapper;
    @Autowired
    private GreenwaveInfoMapper greenwaveInfoMapper;

    public void selectCountByCongestionPeriod() throws ParseException {
        LocalDate currentDate = LocalDate.now();
        // 获取本周一的日期
        LocalDate mondayOfThisWeek = currentDate.with(TemporalAdjusters.previousOrSame(DayOfWeek.MONDAY));
        // 获取上周一的日期
        LocalDate todayOfWeek = mondayOfThisWeek.plusDays(-7);
        //获取上周日的日期
        LocalDate endOfWeek = mondayOfThisWeek.plusDays(-1);
        WeekFields weekFields = WeekFields.of(Locale.getDefault());
        //获取上周的周数
        int Number = todayOfWeek.get(weekFields.weekOfYear());
        int year = currentDate.getYear();
        String weekNumber = String.valueOf(year) + String.valueOf(Number);
        //循环7天处理数据
        for (int i = 0; i < 7; i++) {
            LocalDate date = todayOfWeek.plusDays(i);
            //获取干线数据
            long st = System.currentTimeMillis();
            List<AnalysisGreenCongestionPeriod> greenList = analysisGreenCongestionPeriodMapper.selectGreenDataHist(date.toString());
            long et = System.currentTimeMillis();
            log.info("获取干线数据耗时：{}ms,date:{},size:{}",et-st,date.toString(),greenList.size());
            LinkedList<AnalysisGreenCongestionPeriod> queue = new LinkedList<>();
            String greenId = "";
            String roadDirection = "";
            String startTime = "";
            String endTime = "";
            for (AnalysisGreenCongestionPeriod temp : greenList) {
                if (temp.getGreenId().equals(greenId) && temp.getRoadDirection().equals(roadDirection)) {
                    if (temp.getStatus().equals("2")) {
                        List<AnalysisGreenCongestionPeriod> list = queue.stream().filter(item -> item.getStatus().equals("3")).collect(Collectors.toList());
                        if (!CollectionUtils.isNotEmpty(list)) {
                            List<AnalysisGreenCongestionPeriod> list1 = queue.stream().filter(item -> item.getStatus().equals("2")).collect(Collectors.toList());
                            if (!CollectionUtils.isNotEmpty(list1)) {
                                startTime = temp.getStartTime();
                            }
                        } else {
                            if (list.size() == 1 && queue.getFirst().getStatus().equals("3")) {
                                endTime = queue.getFirst().getStartTime();
                                Map<String, Object> map = new HashMap<>();
                                map.put("startTime", startTime);
                                map.put("endTime", endTime);
                                map.put("weekNumber", weekNumber);
                                map.put("weekDay", i + 1);
                                map.put("roadDirection", roadDirection);
                                map.put("peakType", isMoning(startTime));
                                map.put("weekStartTime", todayOfWeek.toString());
                                map.put("weekEndTime", endOfWeek.toString());
                                map.put("greenId", greenId);
                                map.put("status", queue.getFirst().getStatus());
                                //写入缓行数据
                                analysisGreenCongestionPeriodMapper.insertGreenCongestionPeriodData(map);
                                //记录缓行开始时间
                                if (queue.get(1).getStatus().equals("2")) {
                                    startTime = queue.get(1).getStartTime();
                                } else if (queue.getLast().getStatus().equals("2")) {
                                    startTime = queue.getLast().getStartTime();
                                } else {
                                    startTime = temp.getStartTime();
                                }
                            }
                        }
                    } else if (temp.getStatus().equals("1")) {
                        List<AnalysisGreenCongestionPeriod> list = queue.stream().filter(item -> item.getStatus().equals("3")).collect(Collectors.toList());
                        if (CollectionUtils.isNotEmpty(list) && queue.size() > 2 && list.size() == 1 && queue.getFirst().getStatus().equals("3")) {
                            endTime = queue.getFirst().getStartTime();
                            Map<String, Object> map = new HashMap<>();
                            map.put("startTime", startTime);
                            map.put("endTime", endTime);
                            map.put("weekNumber", weekNumber);
                            map.put("weekDay", i + 1);
                            map.put("roadDirection", roadDirection);
                            map.put("peakType", isMoning(startTime));
                            map.put("weekStartTime", todayOfWeek.toString());
                            map.put("weekEndTime", endOfWeek.toString());
                            map.put("greenId", greenId);
                            map.put("status", queue.getFirst().getStatus());
                            //写入缓行数据
                            try {
                                analysisGreenCongestionPeriodMapper.insertGreenCongestionPeriodData(map);
                            }catch (Exception e){
                                log.error("写入缓行数据异常",e);
                            }

                            if (queue.get(1).getStatus().equals("2")) {
                                startTime = queue.get(1).getStartTime();
                            }
                        } else {
                            List<AnalysisGreenCongestionPeriod> list1 = queue.stream().filter(item -> item.getStatus().equals("2")).collect(Collectors.toList());
                            if (CollectionUtils.isNotEmpty(list1) && queue.size() > 2 && list1.size() == 1 && queue.getFirst().getStatus().equals("2") && !queue.get(1).getStatus().equals("3")) {
                                endTime = queue.getFirst().getStartTime();
                                Map<String, Object> map = new HashMap<>();
                                map.put("startTime", startTime);
                                map.put("endTime", endTime);
                                map.put("weekNumber", weekNumber);
                                map.put("weekDay", i + 1);
                                map.put("roadDirection", roadDirection);
                                map.put("peakType", isMoning(startTime));
                                map.put("weekStartTime", todayOfWeek.toString());
                                map.put("weekEndTime", endOfWeek.toString());
                                map.put("greenId", greenId);
                                map.put("status", queue.getFirst().getStatus());

                                //写入缓行数据
                                try {
                                    analysisGreenCongestionPeriodMapper.insertGreenCongestionPeriodData(map);
                                }catch (Exception e){
                                    log.error("写入缓行数据异常",e);
                                }
                            }
                        }
                    } else if (temp.getStatus().equals("3")) {
                        List<AnalysisGreenCongestionPeriod> list = queue.stream().filter(item -> item.getStatus().equals("3")).collect(Collectors.toList());
                        if (!CollectionUtils.isNotEmpty(list)) {
                            List<AnalysisGreenCongestionPeriod> list1 = queue.stream().filter(item -> item.getStatus().equals("2")).collect(Collectors.toList());
                            if (CollectionUtils.isNotEmpty(list1) && list1.size() >= 1 && queue.getLast().getStatus().equals("2")) {
                                endTime = queue.getLast().getStartTime();
                                Map<String, Object> map = new HashMap<>();
                                map.put("startTime", startTime);
                                map.put("endTime", endTime);
                                map.put("weekNumber", weekNumber);
                                map.put("weekDay", i + 1);
                                map.put("roadDirection", roadDirection);
                                map.put("peakType", isMoning(startTime));
                                map.put("weekStartTime", todayOfWeek.toString());
                                map.put("weekEndTime", endOfWeek.toString());
                                map.put("greenId", greenId);
                                map.put("status", queue.getLast().getStatus());
                                //写入缓行数据
                                try {
                                    analysisGreenCongestionPeriodMapper.insertGreenCongestionPeriodData(map);
                                }catch (Exception e){
                                    log.error("写入缓行数据异常",e);
                                }
                                //记录拥堵开始时间
                                startTime = temp.getStartTime();
                            } else {
                                startTime = temp.getStartTime();
                            }
                        }
                    }
                    enqueue(queue, temp);
                } else if (temp.getGreenId().equals(greenId) && !temp.getRoadDirection().equals(roadDirection)) {
                    queue.clear();
                    roadDirection = temp.getRoadDirection();
                    if (temp.getStatus().equals("2")) {
                        startTime = temp.getStartTime();
                    }
                    enqueue(queue, temp);
                } else {
                    queue.clear();
                    greenId = temp.getGreenId();
                    roadDirection = temp.getRoadDirection();
                    if (temp.getStatus().equals("2")) {
                        startTime = temp.getStartTime();
                    }
                    enqueue(queue, temp);
                }
            }
            long et1 = System.currentTimeMillis();
            log.info("干线数据计算总耗时：{}ms,date:{}",et1-et,date.toString());

        }
    }

    @Override
    public GreenReportProblemOverallAnalysisResult selectListByWeek(Integer year, Integer week, String ids) {
        Integer yearWeek = Integer.valueOf(year + "" + week);
        List<Integer> greenIdList = new ArrayList<>();
        if(ObjectUtils.isEmpty(ids)){
            List<GreenwaveInfoPO> greenwaveInfoPOList = greenwaveInfoMapper.selectAll();
            greenIdList = greenwaveInfoPOList.stream().map(GreenwaveInfoPO::getId).collect(Collectors.toList());
        }else {
            greenIdList = Arrays.stream(ids.split(","))
                    .map(String::trim) // 去除可能的空格
                    .filter(s -> !s.isEmpty()) // 过滤掉空字符串
                    .map(Integer::parseInt) // 转换为整数
                    .collect(Collectors.toList());
        }
        //周干线问题分析
        List<AnalysisGreenCongestionPeriodVO> analysisGreenCongestionPeriodVOS = analysisGreenCongestionPeriodMapper.selectListByWeek(yearWeek);

        GreenReportProblemOverallAnalysisResult greenReportProblemOverallAnalysisResult = new GreenReportProblemOverallAnalysisResult();

        long countTime = 0;
        //筛选拥堵
        List<AnalysisGreenCongestionPeriodVO> voList = analysisGreenCongestionPeriodVOS.stream().filter(x -> x.getStatus() == 3).collect(Collectors.toList());
        //按干线分组
        Map<Integer, List<AnalysisGreenCongestionPeriodVO>> listMap = voList.stream().collect(Collectors.groupingBy(AnalysisGreenCongestionPeriodVO::getGreenId));
        List<GreenReportProblemOverallDataVO> greenList = new ArrayList<>();
        //干线循环
        for (Integer greenId : greenIdList) {
            List<AnalysisGreenCongestionPeriodVO> periodVOS = listMap.get(greenId);
            //按方向分组
            if(ObjectUtils.isEmpty(periodVOS)){
                continue;
            }
            Map<String, List<AnalysisGreenCongestionPeriodVO>> dirMap = periodVOS.stream().collect(Collectors.groupingBy(AnalysisGreenCongestionPeriodVO::getRoadDirection));
            //获取干线信息
            List<GreenWaveCrossRidInfo> greenInfoList = greenWaveCrossRidMapper.getGreenInfoList(String.valueOf(greenId));

            GreenReportProblemOverallDataVO greenReportProblemOverallData = new GreenReportProblemOverallDataVO();
            greenReportProblemOverallData.setGreenId(String.valueOf(greenId));
            greenReportProblemOverallData.setGreenName(greenInfoList.get(0).getGreenwaveName());
            List<GreenWaveDirProblemDataVO> dataList = new ArrayList<>();
            //方向循环
            for (List<AnalysisGreenCongestionPeriodVO> congestionPeriodVOS : dirMap.values()) {
                GreenWaveDirProblemDataVO greenWaveDirProblemData = new GreenWaveDirProblemDataVO();
                greenWaveDirProblemData.setDir(congestionPeriodVOS.get(0).getRoadDirection());
                greenWaveDirProblemData.setCongestCount(congestionPeriodVOS.size());
                greenWaveDirProblemData.setGreenWaveType(RoadLevelEnum.getDesc(greenInfoList.get(0).getLevel()));
                greenWaveDirProblemData.setDir(GreenBeltDirEnum.getDesc(congestionPeriodVOS.get(0).getRoadDirection()));

                long dirCountTime = 0;
                double congestIndexCount = 0;
                for (AnalysisGreenCongestionPeriodVO congestionPeriodVO : congestionPeriodVOS) {
                    congestIndexCount += congestionPeriodVO.getCongestIndex();
                    long time = congestionPeriodVO.getCongestEndTime().getTime() - congestionPeriodVO.getCongestStartTime().getTime();
                    dirCountTime += time;
                    countTime += time;
                }
                long dirHour = dirCountTime / 1000;
                double congestIndexCountAvg = 0;
                if (congestionPeriodVOS.size() != 0) {
                    congestIndexCountAvg = Math.round(congestIndexCount / congestionPeriodVOS.size()*100)/100.0;
                }
                greenWaveDirProblemData.setCongestIndex(congestIndexCountAvg);
                greenWaveDirProblemData.setCongestDuration(Integer.valueOf(String.valueOf(dirHour)));
                dataList.add(greenWaveDirProblemData);
            }
            greenReportProblemOverallData.setList(dataList);
            greenList.add(greenReportProblemOverallData);
        }
        //按拥堵指数降序排序
        List<AnalysisGreenCongestionPeriodVO> periodVOS = voList.stream().sorted(Comparator.comparing(AnalysisGreenCongestionPeriodVO::getCongestIndex).reversed())
                .collect(Collectors.toList());
        Set<Integer> greenIds = new HashSet<>();
        for (AnalysisGreenCongestionPeriodVO periodVO : periodVOS) {
            greenIds.add(periodVO.getGreenId());
            if (greenIds.size() >= 2) {
                break;
            }
        }
        //拼接干线名称
        String congestHeavyGreenWave = "";
        for (Integer greenId : greenIds) {
            List<GreenWaveCrossRidInfo> greenInfoList = greenWaveCrossRidMapper.getGreenInfoList(String.valueOf(greenId));
            if ("".equals(congestHeavyGreenWave)) {
                congestHeavyGreenWave = congestHeavyGreenWave + greenInfoList.get(0).getGreenwaveName();
            } else {
                congestHeavyGreenWave = congestHeavyGreenWave + "、" + greenInfoList.get(0).getGreenwaveName();
            }
        }
        List<GreenReportProblemOverallDataVO> overallDataVOS = greenList.stream().sorted(Comparator.comparing(x -> {
            double congestIndexAll = 0;
            for (GreenWaveDirProblemDataVO dataVO : x.getList()) {
                congestIndexAll += dataVO.getCongestIndex();
            }
            return congestIndexAll;
        })).collect(Collectors.toList());
        for (int i = 0; i < overallDataVOS.size(); i++) {
            overallDataVOS.get(i).setSort(String.valueOf(i+1));
        }
        greenReportProblemOverallAnalysisResult.setCongestHeavyGreenWave(congestHeavyGreenWave);
        greenReportProblemOverallAnalysisResult.setTotalCongestCount(String.valueOf(voList.size())+"次");
        greenReportProblemOverallAnalysisResult.setTotalCongestDuration(String.valueOf(countTime / 60 / 60 / 1000.0)+"小时");
        greenReportProblemOverallAnalysisResult.setDataList(overallDataVOS);

        return greenReportProblemOverallAnalysisResult;
    }

    private SimpleDateFormat format = new SimpleDateFormat("HH:mm");

    @Override
    public List<GreenReportProblemDetailAnalysisResult> getProblemDetail(Integer year, Integer week, String ids) {
        Integer yearWeek = Integer.valueOf(year + "" + week);
        List<Integer> greenIdList = new ArrayList<>();
        if(ObjectUtils.isEmpty(ids)){
            List<GreenwaveInfoPO> greenwaveInfoPOList = greenwaveInfoMapper.selectAll();
            greenIdList = greenwaveInfoPOList.stream().map(GreenwaveInfoPO::getId).collect(Collectors.toList());
        }else {
            greenIdList = Arrays.stream(ids.split(","))
                    .map(String::trim) // 去除可能的空格
                    .filter(s -> !s.isEmpty()) // 过滤掉空字符串
                    .map(Integer::parseInt) // 转换为整数
                    .collect(Collectors.toList());
        }
        //周干线问题分析
        List<AnalysisGreenCongestionPeriodVO> analysisGreenCongestionPeriodVOS = analysisGreenCongestionPeriodMapper.selectListByWeek(yearWeek);
        //按干线分组
        Map<Integer, List<AnalysisGreenCongestionPeriodVO>> greenMap = analysisGreenCongestionPeriodVOS.stream().collect(Collectors.groupingBy(AnalysisGreenCongestionPeriodVO::getGreenId));
        List<GreenReportProblemDetailAnalysisResult> resultList = new ArrayList<>();
        for (Integer greenId : greenIdList) {
            List<AnalysisGreenCongestionPeriodVO> greenCongestionPeriodVOS = greenMap.get(greenId);
            GreenReportProblemDetailAnalysisResult greenReportProblemDetailAnalysisResponse = new GreenReportProblemDetailAnalysisResult();


            List<String> workDayCongestSpanList = new ArrayList<>();
            double workDayCongestTime = 0;
            double workDayCongestIndex = 0;
            int workDayCongestCount = 0;

            List<String> workDaySlowRunSpanList = new ArrayList<>();
            double workDaySlowRunTime = 0;
            double workDaySlowCongestIndex = 0;
            int workDaySlowCongestCount = 0;

            List<String> weekEndCongestSpanList = new ArrayList<>();
            double weekEndCongestTime = 0;
            double weekEndCongestIndex = 0;
            int weekEndCongestCount = 0;

            List<String> weekEndSlowRunSpanList = new ArrayList<>();
            double weekEndSlowRunTime = 0;
            double weekEndSlowCongestIndex = 0;
            int weekEndSlowCongestCount = 0;
            List<GreenReportProblemDetailVO> greenReportProblemDetailList = new ArrayList<>();
            for (AnalysisGreenCongestionPeriodVO periodVO : greenCongestionPeriodVOS) {
                if (periodVO.getStatus() == ConsgestionStatusEnum.CONSGESTION_STATUS.getCode() && periodVO.getWeekDay() <= WeekEnum.FRI.getCode()) {
                    workDayCongestCount++;
                    workDayCongestTime += (periodVO.getCongestEndTime().getTime() - periodVO.getCongestStartTime().getTime());
                    workDayCongestIndex += periodVO.getCongestIndex();
                    String time = getTimeString(periodVO);
                    workDayCongestSpanList.add(time);
                    if (periodVO.getCongestEndTime().getTime() - periodVO.getCongestStartTime().getTime() >= 10 * 60 * 1000) {
                        GreenReportProblemDetailVO greenReportProblemDetail = getGreenReportProblemDetail(periodVO);
                        greenReportProblemDetailList.add(greenReportProblemDetail);
                    }

                } else if (periodVO.getStatus() == ConsgestionStatusEnum.CONSGESTION_STATUS.getCode() && periodVO.getWeekDay() > WeekEnum.FRI.getCode()) {
                    weekEndCongestCount++;
                    weekEndCongestTime += (periodVO.getCongestEndTime().getTime() - periodVO.getCongestStartTime().getTime());
                    weekEndCongestIndex += periodVO.getCongestIndex();
                    String time = getTimeString(periodVO);
                    weekEndCongestSpanList.add(time);
                    if (periodVO.getCongestEndTime().getTime() - periodVO.getCongestStartTime().getTime() >= 10 * 60 * 1000) {
                        GreenReportProblemDetailVO greenReportProblemDetail = getGreenReportProblemDetail(periodVO);
                        greenReportProblemDetailList.add(greenReportProblemDetail);
                    }
                } else if (periodVO.getStatus() == ConsgestionStatusEnum.AMBLE_STATUS.getCode() && periodVO.getWeekDay() <= WeekEnum.FRI.getCode()) {
                    workDaySlowCongestCount++;
                    workDaySlowRunTime += (periodVO.getCongestEndTime().getTime() - periodVO.getCongestStartTime().getTime());
                    workDaySlowCongestIndex += periodVO.getCongestIndex();
                    String time = getTimeString(periodVO);
                    workDaySlowRunSpanList.add(time);
                    if (periodVO.getCongestEndTime().getTime() - periodVO.getCongestStartTime().getTime() >= 10 * 60 * 1000) {
                        GreenReportProblemDetailVO greenReportProblemDetail = getGreenReportProblemDetail(periodVO);
                        greenReportProblemDetailList.add(greenReportProblemDetail);
                    }
                } else if (periodVO.getStatus() == ConsgestionStatusEnum.AMBLE_STATUS.getCode() && periodVO.getWeekDay() > WeekEnum.FRI.getCode()) {
                    weekEndSlowCongestCount++;
                    weekEndSlowRunTime += (periodVO.getCongestEndTime().getTime() - periodVO.getCongestStartTime().getTime());
                    weekEndSlowCongestIndex += periodVO.getCongestIndex();
                    String time = getTimeString(periodVO);
                    weekEndSlowRunSpanList.add(time);
                    if (periodVO.getCongestEndTime().getTime() - periodVO.getCongestStartTime().getTime() >= 10 * 60 * 1000) {
                        GreenReportProblemDetailVO greenReportProblemDetail = getGreenReportProblemDetail(periodVO);
                        greenReportProblemDetailList.add(greenReportProblemDetail);
                    }
                }
            }
            greenReportProblemDetailAnalysisResponse.setDataList(greenReportProblemDetailList);
            List<GreenWaveCrossRidInfo> greenInfoList = greenWaveCrossRidMapper.getGreenInfoList(String.valueOf(greenId));

            greenReportProblemDetailAnalysisResponse.setGreenId(String.valueOf(greenId));
            greenReportProblemDetailAnalysisResponse.setGreenName(greenInfoList.get(0).getGreenwaveName());
            greenReportProblemDetailAnalysisResponse.setWorkDayCongestSpan(TimeIntervalMerger.timeString(workDayCongestSpanList));
            greenReportProblemDetailAnalysisResponse.setWorkDaySlowRunSpan(TimeIntervalMerger.timeString(workDaySlowRunSpanList));
            greenReportProblemDetailAnalysisResponse.setWeekEndCongestSpan(TimeIntervalMerger.timeString(weekEndCongestSpanList));
            greenReportProblemDetailAnalysisResponse.setWeekEndSlowRunSpan(TimeIntervalMerger.timeString(weekEndSlowRunSpanList));
            // 工作日平均拥堵时间
            if (workDayCongestCount != 0) {
                greenReportProblemDetailAnalysisResponse.setWorkDayAvgCongestTime(
                        String.valueOf(Math.round(workDayCongestTime / 1000.0 / 60 / 60 / workDayCongestCount * 100) / 100.0) + "小时"
                );
            } else {
                greenReportProblemDetailAnalysisResponse.setWorkDayAvgCongestTime("0小时");
            }
            // 工作日平均拥堵指数
            if (workDayCongestCount != 0) {
                greenReportProblemDetailAnalysisResponse.setWorkDayAvgCongestIndex(
                        String.valueOf(Math.round(workDayCongestIndex / workDayCongestCount * 100) / 100.0)
                );
            } else {
                greenReportProblemDetailAnalysisResponse.setWorkDayAvgCongestIndex("0");
            }
            // 工作日平均缓行时间
            if (workDaySlowCongestCount != 0) {
                greenReportProblemDetailAnalysisResponse.setWorkDayAvgSlowRunTime(
                        String.valueOf(Math.round(workDaySlowRunTime / 1000.0 / 60 / 60 / workDaySlowCongestCount * 100) / 100.0) + "小时"
                );
            } else {
                greenReportProblemDetailAnalysisResponse.setWorkDayAvgSlowRunTime("0小时");
            }
            // 工作日平均缓行拥堵指数
            if (workDaySlowCongestCount != 0) {
                greenReportProblemDetailAnalysisResponse.setWorkDayAvgSlowAvgCongestIndex(
                        String.valueOf(Math.round(workDaySlowCongestIndex / workDaySlowCongestCount * 100) / 100.0)
                );
            } else {
                greenReportProblemDetailAnalysisResponse.setWorkDayAvgSlowAvgCongestIndex("0");
            }
            // 周末平均拥堵时间
            if (weekEndCongestCount != 0) {
                greenReportProblemDetailAnalysisResponse.setWeekEndAvgCongestTime(
                        String.valueOf(Math.round(weekEndCongestTime / 1000.0 / 60 / 60 / weekEndCongestCount * 100) / 100.0) + "小时"
                );
            } else {
                greenReportProblemDetailAnalysisResponse.setWeekEndAvgCongestTime("0小时");
            }
            // 周末平均拥堵指数
            if (weekEndCongestCount != 0) {
                greenReportProblemDetailAnalysisResponse.setWeekEndAvgCongestIndex(
                        String.valueOf(Math.round(weekEndCongestIndex / weekEndCongestCount * 100) / 100.0)
                );
            } else {
                greenReportProblemDetailAnalysisResponse.setWeekEndAvgCongestIndex("0");
            }
            // 周末平均缓行时间
            if (weekEndSlowCongestCount != 0) {
                greenReportProblemDetailAnalysisResponse.setWeekEndAvgSlowRunTime(
                        String.valueOf(Math.round(weekEndSlowRunTime / 1000.0 / 60 / 60 / weekEndSlowCongestCount * 100) / 100.0) + "小时"
                );
            } else {
                greenReportProblemDetailAnalysisResponse.setWeekEndAvgSlowRunTime("0小时");
            }
            // 周末平均缓行拥堵指数
            if (weekEndSlowCongestCount != 0) {
                greenReportProblemDetailAnalysisResponse.setWeekEndAvgSlowAvgCongestIndex(
                        String.valueOf(Math.round(weekEndSlowCongestIndex / weekEndSlowCongestCount * 100) / 100.0)
                );
            } else {
                greenReportProblemDetailAnalysisResponse.setWeekEndAvgSlowAvgCongestIndex("0");
            }
            resultList.add(greenReportProblemDetailAnalysisResponse);
        }
        return resultList;
    }

    private GreenReportProblemDetailVO getGreenReportProblemDetail(AnalysisGreenCongestionPeriodVO periodVO) {
        GreenReportProblemDetailVO greenReportProblemDetail = new GreenReportProblemDetailVO();
        greenReportProblemDetail.setWeek(WeekEnum.getDesc(periodVO.getWeekDay()));
        greenReportProblemDetail.setProblemSpan(getTimeString(periodVO));
        greenReportProblemDetail.setStatus(ConsgestionStatusEnum.getDesc(periodVO.getStatus()));
        greenReportProblemDetail.setAvgSpeed(Math.round(periodVO.getSpeed() * 100) / 100.0);
        greenReportProblemDetail.setTravelTime(periodVO.getTravelTime() / 60);
        greenReportProblemDetail.setCongestIndex(Math.round(periodVO.getCongestIndex() * 100) / 100.0);
        return greenReportProblemDetail;
    }

    private String getTimeString(AnalysisGreenCongestionPeriodVO periodVO) {
        String start = format.format(periodVO.getCongestStartTime());
        String end = format.format(periodVO.getCongestEndTime());
        return start + "~" + end;
    }

    public void enqueue(LinkedList<AnalysisGreenCongestionPeriod> queue, AnalysisGreenCongestionPeriod temp) {
        if (queue.size() >= 3) {
            queue.poll();
        }
        queue.offer(temp);
    }

    public void enqueue(LinkedList<laneDataHist> queue, laneDataHist temp) {
        if (queue.size() >= 2) {
            queue.poll();
        }
        queue.offer(temp);
    }

    public Integer isMoning(String date) throws ParseException {
        DateFormat fmt = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date temp = fmt.parse(date);
        int hour = temp.getHours();
        if (hour <= 12) {
            return 1;
        } else {
            return 2;
        }
    }

    public void selectCountByLandData() throws ParseException {
        LocalDate currentDate = LocalDate.now();
        // 获取本周一的日期
        LocalDate mondayOfThisWeek = currentDate.with(TemporalAdjusters.previousOrSame(DayOfWeek.MONDAY));
        // 获取上周一的日期
        LocalDate todayOfWeek = mondayOfThisWeek.plusDays(-7);
        //获取上周日的日期
        LocalDate endOfWeek = mondayOfThisWeek.plusDays(-1);
        WeekFields weekFields = WeekFields.of(Locale.getDefault());
        //获取上周的周数
        int Number = todayOfWeek.get(weekFields.weekOfYear());
        int year = currentDate.getYear();
        String weekNumber = String.valueOf(year) + String.valueOf(Number);
        //循环7天处理数据
        for (int i = 0; i < 7; i++) {
            LocalDate date = todayOfWeek.plusDays(i);
            //获取干线数据
            List<laneDataHist> laneList = analysisGreenCongestionPeriodMapper.selectGreenTrafficHist(date.toString());
            LinkedList<laneDataHist> queue = new LinkedList<>();
            LinkedList<laneDataHist> queueCorss = new LinkedList<>();
            List<laneDataHist> list = new ArrayList<>();
            String greenId = "";
            String roadDirection = "";
            String startTime = "";
            String endTime = "";
            Double trafficThreshold = 0.0;
            for(laneDataHist temp : laneList){
                if(temp.getGreenId().equals(greenId)){
                    if(queue.size() == 2){
                        //判断流量是否大于阀值
                        if(trafficThreshold < temp.getTrafficIndex() && com.github.pagehelper.util.StringUtil.isEmpty(startTime)){
                            startTime = temp.getStartTime();
                            list.add(temp);
                        } else if( temp.getTrafficIndex() < trafficThreshold &&
                                queue.getFirst().getTrafficIndex() < trafficThreshold &&
                                queue.getLast().getTrafficIndex() < trafficThreshold &&
                                !com.github.pagehelper.util.StringUtil.isEmpty(startTime)  ){
                            if(list.size() < 10){
                                startTime = "";
                                list.clear();
                                enqueue(queue,temp);
                                continue;
                            }
                            endTime = queue.getFirst().getStartTime();
                            //查询流量数据，循环处理list中的数据找出最大值，并算出平均值
                            List<laneDataHist> crossFlowList = analysisGreenCongestionPeriodMapper.selectCrossFlow(date.toString().replaceAll("-",""),
                                    temp.getCrossId(),startTime,endTime);
                            List<laneDataHist> list1 = new ArrayList<>();
                            Integer maxFlow = 0;
                            Integer sumFlow = 0;
                            if(CollectionUtils.isNotEmpty(crossFlowList)) {
                                for (laneDataHist temp1 : crossFlowList) {
                                    if (queueCorss.size() < 2) {
                                        enqueue(queueCorss, temp1);
                                    } else {
                                        Integer flow = (temp1.getFlow() + queueCorss.getFirst().getFlow() + queueCorss.getLast().getFlow()) * 4 ;
                                        temp1.setFlowHour(flow);
                                        list1.add(temp1);
                                        enqueue(queueCorss, temp1);
                                    }
                                }

                                maxFlow = list1.get(0).getFlowHour();
                                sumFlow = list1.get(0).getFlowHour();
                            }
                            Integer avgFlow = 0;
                            for(int a = 0 ; a < list1.size(); a ++){
                                if(a == list1.size() - 1){
                                    Double avg = Math.ceil((double) sumFlow / list1.size());
                                    avgFlow = Integer.valueOf(avg.intValue()) ;
                                    continue;
                                }
                                if(list1.get(a + 1).getFlowHour() > maxFlow){
                                    maxFlow = list1.get(a + 1).getFlowHour();
                                }
                                sumFlow = sumFlow + list1.get(a + 1).getFlowHour();
                            }
                            //写入数据
                            Map<String,Object> map = new HashMap<>();
                            map.put("startTime",startTime);
                            map.put("endTime",endTime);
                            map.put("weekNumber",weekNumber);
                            map.put("weekDay",i + 1);
                            map.put("roadDirection",roadDirection);
                            map.put("peakType",isMoning(startTime));
                            map.put("weekStartTime",todayOfWeek.toString());
                            map.put("weekEndTime",endOfWeek.toString());
                            map.put("maxFlow",maxFlow);
                            map.put("avgFlow",avgFlow);
                            map.put("greenId",temp.getGreenId());
                            map.put("trafficThreshold",trafficThreshold);
                            analysisGreenCongestionPeriodMapper.insertLaneDataHist(map);
                            //清空数据
                            startTime = "";
                            endTime = "";
                            queueCorss.clear();
                            list.clear();
                        }else {
                            if(!StringUtil.isEmpty(startTime)) {
                                list.add(temp);
                            }
                        }
                    }
                    enqueue(queue,temp);
                }else {
                    queue.clear();
                    list.clear();
                    greenId = temp.getGreenId();
                    roadDirection = temp.getRoadDirection();
                    //计算该干线交通指数阈值，取一天的9点至16点30之间交通指数倒序的90%的数值为动态阈值
                    List<laneDataHist> greenTrafficList = analysisGreenCongestionPeriodMapper.selectGreenTrafficThreshold(date.toString().replaceAll("-",""),greenId,roadDirection);
                    if(greenTrafficList.get(9).getTrafficIndex() <= 1.5) {
                        trafficThreshold = 1.5;
                    }else {
                        trafficThreshold = greenTrafficList.get(9).getTrafficIndex();
                    }
                    startTime = "";
                    endTime = "";
                    enqueue(queue,temp);
                }
            }
            queue.clear();
            greenId = "";
            roadDirection = "";
            startTime = "";
            endTime = "";
            trafficThreshold = 0.0;
            //计算平峰
            List<laneDataHist> peakDetailList = analysisGreenCongestionPeriodMapper.selectPeakDetail(date.toString());
            for(int b = 0 ; b < peakDetailList.size() ; b++){
                if(b == peakDetailList.size() - 1 ){
                   if(queue.size() > 0){
                        if(queue.getFirst().getGreenId().equals(peakDetailList.get(b).getGreenId())){
                            if(!queue.getFirst().getPeakType().equals(peakDetailList.get(b).getPeakType())){
                                startTime = queue.getFirst().getEndTime();
                                endTime = peakDetailList.get(b).getStartTime();
                            }else {
                                startTime = queue.getFirst().getEndTime();
                                endTime = date.toString() + " 20:00:00";
                            }
                        }else {
                            startTime = queue.getFirst().getEndTime();
                            endTime = date.toString() + " 20:00:00";
                            List<laneDataHist> crossFlowList = analysisGreenCongestionPeriodMapper.selectCrossFlow(date.toString().replaceAll("-",""),
                                    queue.getFirst().getCrossId(),startTime,endTime);
                            List<laneDataHist> list1 = new ArrayList<>();
                            Integer maxFlow = 0;
                            Integer sumFlow = 0;
                            if(CollectionUtils.isNotEmpty(crossFlowList)) {
                                for (laneDataHist temp1 : crossFlowList) {
                                    if (queueCorss.size() < 2) {
                                        enqueue(queueCorss, temp1);
                                    } else {
                                        Integer flow = (temp1.getFlow() + queueCorss.getFirst().getFlow() + queueCorss.getLast().getFlow()) * 4 ;
                                        temp1.setFlowHour(flow);
                                        list1.add(temp1);
                                        enqueue(queueCorss, temp1);
                                    }
                                }

                                maxFlow = list1.get(0).getFlowHour();
                                sumFlow = list1.get(0).getFlowHour();
                            }
                            Integer avgFlow = 0;
                            for(int a = 0 ; a < list1.size(); a ++){
                                if(a == list1.size() - 1){
                                    Double avg = Math.ceil((double) sumFlow / list1.size());
                                    avgFlow = Integer.valueOf(avg.intValue()) ;
                                    continue;
                                }
                                if(list1.get(a + 1).getFlowHour() > maxFlow){
                                    maxFlow = list1.get(a + 1).getFlowHour();
                                }
                                sumFlow = sumFlow + list1.get(a + 1).getFlowHour();
                            }
                            //写入数据
                            Map<String,Object> map = new HashMap<>();
                            map.put("startTime",startTime);
                            map.put("endTime",endTime);
                            map.put("weekNumber",weekNumber);
                            map.put("weekDay",i + 1);
                            map.put("roadDirection",roadDirection);
                            map.put("peakType",3);
                            map.put("weekStartTime",todayOfWeek.toString());
                            map.put("weekEndTime",endOfWeek.toString());
                            map.put("maxFlow",maxFlow);
                            map.put("avgFlow",avgFlow);
                            map.put("greenId",queue.getFirst().getGreenId());
                            map.put("trafficThreshold",trafficThreshold);
                            analysisGreenCongestionPeriodMapper.insertLaneDataHist(map);
                            queueCorss.clear();
                            if(peakDetailList.get(b).getPeakType().equals("2")) {
                                startTime = date.toString() + " 08:00:00";
                                endTime = peakDetailList.get(b).getStartTime();

                            }else {
                                startTime = peakDetailList.get(b).getEndTime();
                                endTime = date.toString() + " 20:00:00";
                            }
                        }
                   }else {
                       if(peakDetailList.get(b).getPeakType().equals("2")) {
                           startTime = date.toString() + " 08:00:00";
                           endTime = peakDetailList.get(b).getStartTime();

                       }else {
                           startTime = peakDetailList.get(b).getEndTime();
                           endTime = date.toString() + " 20:00:00";
                       }
                   }
                    List<laneDataHist> crossFlowList = analysisGreenCongestionPeriodMapper.selectCrossFlow(date.toString().replaceAll("-",""),
                            peakDetailList.get(b).getCrossId(),startTime,endTime);
                    List<laneDataHist> list1 = new ArrayList<>();
                    Integer maxFlow = 0;
                    Integer sumFlow = 0;
                    if(CollectionUtils.isNotEmpty(crossFlowList)) {
                        for (laneDataHist temp1 : crossFlowList) {
                            if (queueCorss.size() < 2) {
                                enqueue(queueCorss, temp1);
                            } else {
                                Integer flow = (temp1.getFlow() + queueCorss.getFirst().getFlow() + queueCorss.getLast().getFlow()) * 4 ;
                                temp1.setFlowHour(flow);
                                list1.add(temp1);
                                enqueue(queueCorss, temp1);
                            }
                        }

                        maxFlow = list1.get(0).getFlowHour();
                        sumFlow = list1.get(0).getFlowHour();
                    }
                    Integer avgFlow = 0;
                    for(int a = 0 ; a < list1.size(); a ++){
                        if(a == list1.size() - 1){
                            Double avg = Math.ceil((double) sumFlow / list1.size());
                            avgFlow = Integer.valueOf(avg.intValue()) ;
                            continue;
                        }
                        if(list1.get(a + 1).getFlowHour() > maxFlow){
                            maxFlow = list1.get(a + 1).getFlowHour();
                        }
                        sumFlow = sumFlow + list1.get(a + 1).getFlowHour();
                    }
                    //写入数据
                    Map<String,Object> map = new HashMap<>();
                    map.put("startTime",startTime);
                    map.put("endTime",endTime);
                    map.put("weekNumber",weekNumber);
                    map.put("weekDay",i + 1);
                    map.put("roadDirection",roadDirection);
                    map.put("peakType",3);
                    map.put("weekStartTime",todayOfWeek.toString());
                    map.put("weekEndTime",endOfWeek.toString());
                    map.put("maxFlow",maxFlow);
                    map.put("avgFlow",avgFlow);
                    map.put("greenId",peakDetailList.get(b).getGreenId());
                    map.put("trafficThreshold",trafficThreshold);
                    analysisGreenCongestionPeriodMapper.insertLaneDataHist(map);
                    startTime = "";
                    endTime = "";
                    queue.clear();
                    continue;

                }
                if(peakDetailList.get(b).getGreenId().equals(greenId)){
                    if(queue.size() > 0) {
                        if (peakDetailList.get(b).getPeakType().equals("2") && queue.getFirst().getPeakType().equals("1")) {
                            startTime = queue.getFirst().getEndTime();
                            endTime = peakDetailList.get(b).getStartTime();
                            List<laneDataHist> crossFlowList = analysisGreenCongestionPeriodMapper.selectCrossFlow(date.toString().replaceAll("-", ""),
                                    peakDetailList.get(b).getCrossId(), startTime, endTime);
                            List<laneDataHist> list1 = new ArrayList<>();
                            Integer maxFlow = 0;
                            Integer sumFlow = 0;
                            if (CollectionUtils.isNotEmpty(crossFlowList)) {
                                for (laneDataHist temp1 : crossFlowList) {
                                    if (queueCorss.size() < 2) {
                                        enqueue(queueCorss, temp1);
                                    } else {
                                        Integer flow = (temp1.getFlow() + queueCorss.getFirst().getFlow() + queueCorss.getLast().getFlow()) * 4;
                                        temp1.setFlowHour(flow);
                                        list1.add(temp1);
                                        enqueue(queueCorss, temp1);
                                    }
                                }

                                maxFlow = list1.get(0).getFlowHour();
                                sumFlow = list1.get(0).getFlowHour();
                            }
                            Integer avgFlow = 0;
                            for (int a = 0; a < list1.size(); a++) {
                                if (a == list1.size() - 1) {
                                    Double avg = Math.ceil((double) sumFlow / list1.size());
                                    avgFlow = Integer.valueOf(avg.intValue());
                                    continue;
                                }
                                if (list1.get(a + 1).getFlowHour() > maxFlow) {
                                    maxFlow = list1.get(a + 1).getFlowHour();
                                }
                                sumFlow = sumFlow + list1.get(a + 1).getFlowHour();
                            }
                            //写入数据
                            Map<String, Object> map = new HashMap<>();
                            map.put("startTime", startTime);
                            map.put("endTime", endTime);
                            map.put("weekNumber", weekNumber);
                            map.put("weekDay", i + 1);
                            map.put("roadDirection", roadDirection);
                            map.put("peakType", 3);
                            map.put("weekStartTime", todayOfWeek.toString());
                            map.put("weekEndTime", endOfWeek.toString());
                            map.put("maxFlow", maxFlow);
                            map.put("avgFlow", avgFlow);
                            map.put("greenId", peakDetailList.get(b).getGreenId());
                            map.put("trafficThreshold", trafficThreshold);
                            analysisGreenCongestionPeriodMapper.insertLaneDataHist(map);
                            startTime = "";
                            endTime = "";
                            queueCorss.clear();
                            queue.clear();
                        }
                    }
                }else {
                    if(queue.size() == 1){
                        if(queue.getFirst().getPeakType().equals("1")) {
                            startTime = queue.getFirst().getEndTime();
                            endTime = date.toString() + " 20:00:00";
                        }else {
                            startTime = date.toString() + " 08:00:00";
                            endTime = queue.getFirst().getStartTime();
                        }
                        List<laneDataHist> crossFlowList = analysisGreenCongestionPeriodMapper.selectCrossFlow(date.toString().replaceAll("-",""),
                                queue.getFirst().getCrossId(),startTime,endTime);
                        List<laneDataHist> list1 = new ArrayList<>();
                        Integer maxFlow = 0;
                        Integer sumFlow = 0;
                        if(CollectionUtils.isNotEmpty(crossFlowList)) {
                            for (laneDataHist temp1 : crossFlowList) {
                                if (queueCorss.size() < 2) {
                                    enqueue(queueCorss, temp1);
                                } else {
                                    Integer flow = (temp1.getFlow() + queueCorss.getFirst().getFlow() + queueCorss.getLast().getFlow()) * 4 ;
                                    temp1.setFlowHour(flow);
                                    list1.add(temp1);
                                    enqueue(queueCorss, temp1);
                                }
                            }

                            maxFlow = list1.get(0).getFlowHour();
                            sumFlow = list1.get(0).getFlowHour();
                        }
                        Integer avgFlow = 0;
                        for(int a = 0 ; a < list1.size(); a ++){
                            if(a == list1.size() - 1){
                                Double avg = Math.ceil((double) sumFlow / list1.size());
                                avgFlow = Integer.valueOf(avg.intValue()) ;
                                continue;
                            }
                            if(list1.get(a + 1).getFlowHour() > maxFlow){
                                maxFlow = list1.get(a + 1).getFlowHour();
                            }
                            sumFlow = sumFlow + list1.get(a + 1).getFlowHour();
                        }
                        //写入数据
                        Map<String,Object> map = new HashMap<>();
                        map.put("startTime",startTime);
                        map.put("endTime",endTime);
                        map.put("weekNumber",weekNumber);
                        map.put("weekDay",i + 1);
                        map.put("roadDirection",roadDirection);
                        map.put("peakType",3);
                        map.put("weekStartTime",todayOfWeek.toString());
                        map.put("weekEndTime",endOfWeek.toString());
                        map.put("maxFlow",maxFlow);
                        map.put("avgFlow",avgFlow);
                        map.put("greenId",queue.getFirst().getGreenId());
                        map.put("trafficThreshold",trafficThreshold);
                        analysisGreenCongestionPeriodMapper.insertLaneDataHist(map);
                        startTime = "";
                        endTime = "";
                        queueCorss.clear();
                        queue.clear();
                    }
                    queue.clear();
                    greenId = peakDetailList.get(b).getGreenId();
                    roadDirection = peakDetailList.get(b).getRoadDirection();
                    enqueue(queue,peakDetailList.get(b));
                }
            }
        }
    }

    public void selectCountByCrossData() throws ParseException {
        LocalDate currentDate = LocalDate.now();
        // 获取本周一的日期
        LocalDate mondayOfThisWeek = currentDate.with(TemporalAdjusters.previousOrSame(DayOfWeek.MONDAY));
        // 获取上周一的日期
        LocalDate todayOfWeek = mondayOfThisWeek.plusDays(-7);
        //获取上周日的日期
        LocalDate endOfWeek = mondayOfThisWeek.plusDays(-1);
        WeekFields weekFields = WeekFields.of(Locale.getDefault());
        //获取上周的周数
        int Number = todayOfWeek.get(weekFields.weekOfYear());
        int year = currentDate.getYear();
        String weekNumber = String.valueOf(year) + String.valueOf(Number);
        //循环7天处理数据
        for (int i = 0; i < 7; i++) {
            LocalDate date = todayOfWeek.plusDays(i);
            //获取干线数据
            List<laneDataHist> crossList = analysisGreenCongestionPeriodMapper.selectCrossTrafficHist(date.toString().replaceAll("-",""));
            LinkedList<laneDataHist> queue = new LinkedList<>();
            LinkedList<laneDataHist> queueCorss = new LinkedList<>();
            List<laneDataHist> list = new ArrayList<>();
            String crossId = "";
            String startTime = "";
            String endTime = "";
            Double trafficThreshold = 0.0;
            for(laneDataHist temp : crossList){
                if(temp.getCrossId().equals(crossId)){
                    if(queue.size() == 2){
                        //判断流量是否大于阀值
                        if(trafficThreshold < temp.getTrafficIndex() && com.github.pagehelper.util.StringUtil.isEmpty(startTime)){
                            startTime = temp.getStartTime();
                            list.add(temp);
                        } else if( temp.getTrafficIndex() < trafficThreshold &&
                                queue.getFirst().getTrafficIndex() < trafficThreshold &&
                                queue.getLast().getTrafficIndex() < trafficThreshold &&
                                !com.github.pagehelper.util.StringUtil.isEmpty(startTime)  ){
                            if(list.size() < 10){
                                startTime = "";
                                list.clear();
                                enqueue(queue,temp);
                                continue;
                            }
                            endTime = queue.getFirst().getStartTime();
                            //查询流量数据，循环处理list中的数据找出最大值，并算出平均值
                            List<laneDataHist> crossFlowList = analysisGreenCongestionPeriodMapper.selectCrossDataHistFlow(date.toString().replaceAll("-",""),
                                    temp.getCrossId());
                            List<laneDataHist> list1 = new ArrayList<>();
                            Integer maxFlow = 0;
                            Integer maxFlowDate = 0;
                            Integer sumFlowDate = 0;
                            if(CollectionUtils.isNotEmpty(crossFlowList)) {
                                for (laneDataHist temp1 : crossFlowList) {
                                    if (queueCorss.size() < 2) {
                                        enqueue(queueCorss, temp1);
                                    } else {
                                        Integer flow = (temp1.getFlow() + queueCorss.getFirst().getFlow() + queueCorss.getLast().getFlow()) * 4 ;
                                        temp1.setFlowHour(flow);
                                        list1.add(temp1);
                                        enqueue(queueCorss, temp1);
                                    }
                                }

                                maxFlowDate = list1.get(0).getFlowHour();
                                sumFlowDate = list1.get(0).getFlowHour();
                            }

                            for(int a = 0 ; a < list1.size(); a ++){
                                if(a == list1.size() - 1){
                                    continue;
                                }
                                SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                                Date startTimeDate = format.parse(startTime);
                                Date endTimeDate = format.parse(endTime);
                                Date tempdate = format.parse(list1.get(a).getStartTime());
                                if(tempdate.compareTo(startTimeDate) >= 0 && tempdate.compareTo(endTimeDate) < 0 )
                                {
                                    if(list1.get(a + 1).getFlowHour() > list1.get(a).getFlowHour()){
                                        maxFlow = list1.get(a + 1).getFlowHour();
                                    }else {
                                        maxFlow = list1.get(a).getFlowHour();
                                    }
                                }
                                if(list1.get(a + 1).getFlowHour() > maxFlowDate){
                                    maxFlowDate = list1.get(a + 1).getFlowHour();
                                }
                                sumFlowDate = sumFlowDate + list1.get(a + 1).getFlowHour();
                            }
                            //写入数据
                            Map<String,Object> map = new HashMap<>();
                            map.put("startTime",startTime);
                            map.put("endTime",endTime);
                            map.put("weekNumber",weekNumber);
                            map.put("weekDay",i + 1);
                            map.put("peakType",isMoning(startTime));
                            map.put("dayType",isWorkDate(i + 1));
                            map.put("weekStartTime",todayOfWeek);
                            map.put("weekEndTime",endOfWeek);
                            map.put("maxFlow",maxFlow);
                            map.put("maxFlowDate",maxFlowDate);
                            map.put("sumFlowDate",sumFlowDate);
                            map.put("crossId",temp.getCrossId());
                            analysisGreenCongestionPeriodMapper.insertCrossDataHist(map);
                            //清空数据
                            startTime = "";
                            endTime = "";
                            list.clear();
                            queueCorss.clear();
                        }else {
                            if(!StringUtil.isEmpty(startTime)) {
                                list.add(temp);
                            }
                        }
                    }
                    enqueue(queue,temp);
                }else {
                    queue.clear();
                    queueCorss.clear();
                    list.clear();
                    crossId = temp.getCrossId();
                    //计算该路口交通指数阈值，取一天的9点至16点30之间交通指数倒序的90%的数值为动态阈值
                    List<laneDataHist> crossTrafficList = analysisGreenCongestionPeriodMapper.selectCrossTrafficThreshold(date.toString().replaceAll("-",""),crossId);
                    if(crossTrafficList.get(9).getTrafficIndex() <= 1.5) {
                        trafficThreshold = 1.5;
                    }else {
                        trafficThreshold = crossTrafficList.get(9).getTrafficIndex();
                    }
                    startTime = "";
                    endTime = "";
                    enqueue(queue,temp);
                }
            }
            queue.clear();
            queueCorss.clear();
            crossId = "";
            startTime = "";
            endTime = "";
            trafficThreshold = 0.0;
            //计算平峰
            List<laneDataHist> peakDetailList = analysisGreenCongestionPeriodMapper.selectCrossPeakDetail(date.toString());
            for(int b = 0 ; b < peakDetailList.size() ; b++){
                if(b == peakDetailList.size() - 1 ){
                    if(queue.size() > 0){
                        if(queue.getFirst().getCrossId().equals(peakDetailList.get(b).getCrossId())){
                            if(!queue.getFirst().getPeakType().equals(peakDetailList.get(b).getPeakType())){
                                startTime = queue.getFirst().getEndTime();
                                endTime = peakDetailList.get(b).getStartTime();
                            }else {
                                startTime = queue.getFirst().getEndTime();
                                endTime = date.toString() + " 20:00:00";
                            }
                        }else {
                            startTime = queue.getFirst().getEndTime();
                            endTime = date.toString() + " 20:00:00";
                            List<laneDataHist> crossFlowList = analysisGreenCongestionPeriodMapper.selectCrossDataHistFlow(date.toString().replaceAll("-",""),
                                    queue.getFirst().getCrossId());
                            List<laneDataHist> list1 = new ArrayList<>();
                            Integer maxFlow = 0;
                            Integer maxFlowDate = 0;
                            Integer sumFlowDate = 0;
                            if(CollectionUtils.isNotEmpty(crossFlowList)) {
                                for (laneDataHist temp1 : crossFlowList) {
                                    if (queueCorss.size() < 2) {
                                        enqueue(queueCorss, temp1);
                                    } else {
                                        Integer flow = (temp1.getFlow() + queueCorss.getFirst().getFlow() + queueCorss.getLast().getFlow()) * 4 ;
                                        temp1.setFlowHour(flow);
                                        list1.add(temp1);
                                        enqueue(queueCorss, temp1);
                                    }
                                }

                                maxFlowDate = list1.get(0).getFlowHour();
                                sumFlowDate = list1.get(0).getFlowHour();
                            }

                            for(int a = 0 ; a < list1.size(); a ++){
                                if(a == list1.size() - 1){
                                    continue;
                                }
                                SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                                Date startTimeDate = format.parse(startTime);
                                Date endTimeDate = format.parse(endTime);
                                Date tempdate = format.parse(list1.get(a).getStartTime());
                                if(tempdate.compareTo(startTimeDate) >= 0 && tempdate.compareTo(endTimeDate) < 0 )
                                {
                                    if(list1.get(a + 1).getFlowHour() > list1.get(a).getFlowHour()){
                                        maxFlow = list1.get(a + 1).getFlowHour();
                                    }else {
                                        maxFlow = list1.get(a).getFlowHour();
                                    }
                                }
                                if(list1.get(a + 1).getFlowHour() > maxFlowDate){
                                    maxFlowDate = list1.get(a + 1).getFlowHour();
                                }
                                sumFlowDate = sumFlowDate + list1.get(a + 1).getFlowHour();
                            }
                            //写入数据
                            Map<String,Object> map = new HashMap<>();
                            map.put("startTime",startTime);
                            map.put("endTime",endTime);
                            map.put("weekNumber",weekNumber);
                            map.put("weekDay",i + 1);
                            map.put("peakType",3);
                            map.put("dayType",isWorkDate(i + 1));
                            map.put("weekStartTime",todayOfWeek);
                            map.put("weekEndTime",endOfWeek);
                            map.put("maxFlow",maxFlow);
                            map.put("maxFlowDate",maxFlowDate);
                            map.put("sumFlowDate",sumFlowDate);
                            map.put("crossId",queue.getFirst().getCrossId());
                            analysisGreenCongestionPeriodMapper.insertCrossDataHist(map);
                            queueCorss.clear();
                            if(peakDetailList.get(b).getPeakType().equals("2")) {
                                startTime = date.toString() + " 08:00:00";
                                endTime = peakDetailList.get(b).getStartTime();

                            }else {
                                startTime = peakDetailList.get(b).getEndTime();
                                endTime = date.toString() + " 20:00:00";
                            }
                        }
                    }else {
                        if(peakDetailList.get(b).getPeakType().equals("2")) {
                            startTime = date.toString() + " 08:00:00";
                            endTime = peakDetailList.get(b).getStartTime();

                        }else {
                            startTime = peakDetailList.get(b).getEndTime();
                            endTime = date.toString() + " 20:00:00";
                        }
                    }
                    List<laneDataHist> crossFlowList = analysisGreenCongestionPeriodMapper.selectCrossDataHistFlow(date.toString().replaceAll("-",""),
                            peakDetailList.get(b).getCrossId());
                    List<laneDataHist> list1 = new ArrayList<>();
                    Integer maxFlow = 0;
                    Integer maxFlowDate = 0;
                    Integer sumFlowDate = 0;
                    if(CollectionUtils.isNotEmpty(crossFlowList)) {
                        for (laneDataHist temp1 : crossFlowList) {
                            if (queueCorss.size() < 2) {
                                enqueue(queueCorss, temp1);
                            } else {
                                Integer flow = (temp1.getFlow() + queueCorss.getFirst().getFlow() + queueCorss.getLast().getFlow()) * 4 ;
                                temp1.setFlowHour(flow);
                                list1.add(temp1);
                                enqueue(queueCorss, temp1);
                            }
                        }

                        maxFlowDate = list1.get(0).getFlowHour();
                        sumFlowDate = list1.get(0).getFlowHour();
                    }

                    for(int a = 0 ; a < list1.size(); a ++){
                        if(a == list1.size() - 1){
                            continue;
                        }
                        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                        Date startTimeDate = format.parse(startTime);
                        Date endTimeDate = format.parse(endTime);
                        Date tempdate = format.parse(list1.get(a).getStartTime());
                        if(tempdate.compareTo(startTimeDate) >= 0 && tempdate.compareTo(endTimeDate) < 0 )
                        {
                            if(list1.get(a + 1).getFlowHour() > list1.get(a).getFlowHour()){
                                maxFlow = list1.get(a + 1).getFlowHour();
                            }else {
                                maxFlow = list1.get(a).getFlowHour();
                            }
                        }
                        if(list1.get(a + 1).getFlowHour() > maxFlowDate){
                            maxFlowDate = list1.get(a + 1).getFlowHour();
                        }
                        sumFlowDate = sumFlowDate + list1.get(a + 1).getFlowHour();
                    }
                    //写入数据
                    Map<String,Object> map = new HashMap<>();
                    map.put("startTime",startTime);
                    map.put("endTime",endTime);
                    map.put("weekNumber",weekNumber);
                    map.put("weekDay",i + 1);
                    map.put("peakType",3);
                    map.put("dayType",isWorkDate(i + 1));
                    map.put("weekStartTime",todayOfWeek);
                    map.put("weekEndTime",endOfWeek);
                    map.put("maxFlow",maxFlow);
                    map.put("maxFlowDate",maxFlowDate);
                    map.put("sumFlowDate",sumFlowDate);
                    map.put("crossId",peakDetailList.get(b).getCrossId());
                    analysisGreenCongestionPeriodMapper.insertCrossDataHist(map);
                    startTime = "";
                    endTime = "";
                    queue.clear();
                    queueCorss.clear();
                    continue;

                }
                if(peakDetailList.get(b).getCrossId().equals(crossId)){
                    if(queue.size() > 0) {
                        if (peakDetailList.get(b).getPeakType().equals("2") && queue.getFirst().getPeakType().equals("1")) {
                            startTime = queue.getFirst().getEndTime();
                            endTime = peakDetailList.get(b).getStartTime();
                            List<laneDataHist> crossFlowList = analysisGreenCongestionPeriodMapper.selectCrossDataHistFlow(date.toString().replaceAll("-",""),
                                    peakDetailList.get(b).getCrossId());
                            List<laneDataHist> list1 = new ArrayList<>();
                            Integer maxFlow = 0;
                            Integer maxFlowDate = 0;
                            Integer sumFlowDate = 0;
                            if(CollectionUtils.isNotEmpty(crossFlowList)) {
                                for (laneDataHist temp1 : crossFlowList) {
                                    if (queueCorss.size() < 2) {
                                        enqueue(queueCorss, temp1);
                                    } else {
                                        Integer flow = (temp1.getFlow() + queueCorss.getFirst().getFlow() + queueCorss.getLast().getFlow()) * 4 ;
                                        temp1.setFlowHour(flow);
                                        list1.add(temp1);
                                        enqueue(queueCorss, temp1);
                                    }
                                }

                                maxFlowDate = list1.get(0).getFlowHour();
                                sumFlowDate = list1.get(0).getFlowHour();
                            }

                            for(int a = 0 ; a < list1.size(); a ++){
                                if(a == list1.size() - 1){
                                    continue;
                                }
                                SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                                Date startTimeDate = format.parse(startTime);
                                Date endTimeDate = format.parse(endTime);
                                Date tempdate = format.parse(list1.get(a).getStartTime());
                                if(tempdate.compareTo(startTimeDate) >= 0 && tempdate.compareTo(endTimeDate) < 0 )
                                {
                                    if(list1.get(a + 1).getFlowHour() > list1.get(a).getFlowHour()){
                                        maxFlow = list1.get(a + 1).getFlowHour();
                                    }else {
                                        maxFlow = list1.get(a).getFlowHour();
                                    }
                                }
                                if(list1.get(a + 1).getFlowHour() > maxFlowDate){
                                    maxFlowDate = list1.get(a + 1).getFlowHour();
                                }
                                sumFlowDate = sumFlowDate + list1.get(a + 1).getFlowHour();
                            }
                            //写入数据
                            Map<String,Object> map = new HashMap<>();
                            map.put("startTime",startTime);
                            map.put("endTime",endTime);
                            map.put("weekNumber",weekNumber);
                            map.put("weekDay",i + 1);
                            map.put("peakType",3);
                            map.put("dayType",isWorkDate(i + 1));
                            map.put("weekStartTime",todayOfWeek);
                            map.put("weekEndTime",endOfWeek);
                            map.put("maxFlow",maxFlow);
                            map.put("maxFlowDate",maxFlowDate);
                            map.put("sumFlowDate",sumFlowDate);
                            map.put("crossId",peakDetailList.get(b).getCrossId());
                            analysisGreenCongestionPeriodMapper.insertCrossDataHist(map);
                            startTime = "";
                            endTime = "";
                            queue.clear();
                            queueCorss.clear();
                        }
                    }
                }else {
                    if(queue.size() == 1){
                        if(queue.getFirst().getPeakType().equals("1")) {
                            startTime = queue.getFirst().getEndTime();
                            endTime = date.toString() + " 20:00:00";
                        }else {
                            startTime = date.toString() + " 08:00:00";
                            endTime = queue.getFirst().getStartTime();
                        }
                        List<laneDataHist> crossFlowList = analysisGreenCongestionPeriodMapper.selectCrossDataHistFlow(date.toString().replaceAll("-",""),
                                queue.getFirst().getCrossId());
                        List<laneDataHist> list1 = new ArrayList<>();
                        Integer maxFlow = 0;
                        Integer maxFlowDate = 0;
                        Integer sumFlowDate = 0;
                        if(CollectionUtils.isNotEmpty(crossFlowList)) {
                            for (laneDataHist temp1 : crossFlowList) {
                                if (queueCorss.size() < 2) {
                                    enqueue(queueCorss, temp1);
                                } else {
                                    Integer flow = (temp1.getFlow() + queueCorss.getFirst().getFlow() + queueCorss.getLast().getFlow()) * 4 ;
                                    temp1.setFlowHour(flow);
                                    list1.add(temp1);
                                    enqueue(queueCorss, temp1);
                                }
                            }

                            maxFlowDate = list1.get(0).getFlowHour();
                            sumFlowDate = list1.get(0).getFlowHour();
                        }

                        for(int a = 0 ; a < list1.size(); a ++){
                            if(a == list1.size() - 1){
                                continue;
                            }
                            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                            Date startTimeDate = format.parse(startTime);
                            Date endTimeDate = format.parse(endTime);
                            Date tempdate = format.parse(list1.get(a).getStartTime());
                            if(tempdate.compareTo(startTimeDate) >= 0 && tempdate.compareTo(endTimeDate) < 0 )
                            {
                                if(list1.get(a + 1).getFlowHour() > list1.get(a).getFlowHour()){
                                    maxFlow = list1.get(a + 1).getFlowHour();
                                }else {
                                    maxFlow = list1.get(a).getFlowHour();
                                }
                            }
                            if(list1.get(a + 1).getFlowHour() > maxFlowDate){
                                maxFlowDate = list1.get(a + 1).getFlowHour();
                            }
                            sumFlowDate = sumFlowDate + list1.get(a + 1).getFlowHour();
                        }
                        //写入数据
                        Map<String,Object> map = new HashMap<>();
                        map.put("startTime",startTime);
                        map.put("endTime",endTime);
                        map.put("weekNumber",weekNumber);
                        map.put("weekDay",i + 1);
                        map.put("peakType",3);
                        map.put("dayType",isWorkDate(i + 1));
                        map.put("weekStartTime",todayOfWeek);
                        map.put("weekEndTime",endOfWeek);
                        map.put("maxFlow",maxFlow);
                        map.put("maxFlowDate",maxFlowDate);
                        map.put("sumFlowDate",sumFlowDate);
                        map.put("crossId",queue.getFirst().getCrossId());
                        analysisGreenCongestionPeriodMapper.insertCrossDataHist(map);
                        startTime = "";
                        endTime = "";
                        queue.clear();
                        queueCorss.clear();
                    }
                    queue.clear();
                    crossId = peakDetailList.get(b).getCrossId();
                    enqueue(queue,peakDetailList.get(b));
                }
            }
        }

        //更新统计周的通行能力
        //获取统计周的最大流量值
        List<laneDataHist> crossMAXFlowList = analysisGreenCongestionPeriodMapper.selectCrossMAXPeakDetail(weekNumber);
        //循环更新数据
        for(laneDataHist temp : crossMAXFlowList){
            Map<String,Object> map = new HashMap<>();
            map.put("crossId",temp.getCrossId());
            map.put("peakType",temp.getPeakType());
            map.put("weekNumber",weekNumber);
            map.put("maxFlow",temp.getFlow());
            analysisGreenCongestionPeriodMapper.updateCrossMAXPeakDetail(map);
        }

    }

    public Integer isWorkDate(int date) throws ParseException {
        if (date == 6 || date == 7) {
            return 2;
        } else {
            return 1;
        }
    }

    public void selectCountByCrossDataOneDay() throws ParseException {
        LocalDate currentDate = LocalDate.now();
        //昨天
        LocalDate date = currentDate.plusDays(-1);
        WeekFields weekFields = WeekFields.of(Locale.getDefault());
        //获取昨天的周一日期
        LocalDate mondayOfThisWeek = date.with(TemporalAdjusters.previousOrSame(DayOfWeek.MONDAY));
        // 获取周一的日期
        LocalDate todayOfWeek = mondayOfThisWeek;
        //获取周日的日期
        LocalDate endOfWeek = mondayOfThisWeek.plusDays(6);
        //获取昨天的周数
        int Number = date.get(weekFields.weekOfYear());
        int year = date.getYear();
        DayOfWeek dayOfWeek = date.getDayOfWeek();
        int day = dayOfWeek.getValue();
        String weekNumber = String.valueOf(year) + String.valueOf(Number);
        //处理数据昨天数据

        //获取干线数据
        List<laneDataHist> crossList = analysisGreenCongestionPeriodMapper.selectCrossTrafficHist(date.toString().replaceAll("-",""));
        LinkedList<laneDataHist> queue = new LinkedList<>();
        LinkedList<laneDataHist> queueCorss = new LinkedList<>();
        List<laneDataHist> list = new ArrayList<>();
        String crossId = "";
        String startTime = "";
        String endTime = "";
        Double trafficThreshold = 0.0;
        for(laneDataHist temp : crossList){
            if(temp.getCrossId().equals(crossId)){
                if(queue.size() == 2){
                    //判断流量是否大于阀值
                    if(trafficThreshold < temp.getTrafficIndex() && com.github.pagehelper.util.StringUtil.isEmpty(startTime)){
                        startTime = temp.getStartTime();
                        list.add(temp);
                    } else if( temp.getTrafficIndex() < trafficThreshold &&
                            queue.getFirst().getTrafficIndex() < trafficThreshold &&
                            queue.getLast().getTrafficIndex() < trafficThreshold &&
                            !com.github.pagehelper.util.StringUtil.isEmpty(startTime)  ){
                        if(list.size() < 10){
                            startTime = "";
                            list.clear();
                            enqueue(queue,temp);
                            continue;
                        }
                        endTime = queue.getFirst().getStartTime();
                        //查询流量数据，循环处理list中的数据找出最大值，并算出平均值
                        List<laneDataHist> crossFlowList = analysisGreenCongestionPeriodMapper.selectCrossDataHistFlow(date.toString().replaceAll("-",""),
                                temp.getCrossId());
                        List<laneDataHist> list1 = new ArrayList<>();
                        Integer maxFlow = 0;
                        Integer maxFlowDate = 0;
                        Integer sumFlowDate = 0;
                        if(CollectionUtils.isNotEmpty(crossFlowList)) {
                            for (laneDataHist temp1 : crossFlowList) {
                                if (queueCorss.size() < 2) {
                                    enqueue(queueCorss, temp1);
                                } else {
                                    Integer flow = (temp1.getFlow() + queueCorss.getFirst().getFlow() + queueCorss.getLast().getFlow()) * 4 ;
                                    temp1.setFlowHour(flow);
                                    list1.add(temp1);
                                    enqueue(queueCorss, temp1);
                                }
                            }

                            maxFlowDate = list1.get(0).getFlowHour();
                            sumFlowDate = list1.get(0).getFlowHour();
                        }

                        for(int a = 0 ; a < list1.size(); a ++){
                            if(a == list1.size() - 1){
                                continue;
                            }
                            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                            Date startTimeDate = format.parse(startTime);
                            Date endTimeDate = format.parse(endTime);
                            Date tempdate = format.parse(list1.get(a).getStartTime());
                            if(tempdate.compareTo(startTimeDate) >= 0 && tempdate.compareTo(endTimeDate) < 0 )
                            {
                                if(list1.get(a + 1).getFlowHour() > list1.get(a).getFlowHour()){
                                    maxFlow = list1.get(a + 1).getFlowHour();
                                }else {
                                    maxFlow = list1.get(a).getFlowHour();
                                }
                            }
                            if(list1.get(a + 1).getFlowHour() > maxFlowDate){
                                maxFlowDate = list1.get(a + 1).getFlowHour();
                            }
                            sumFlowDate = sumFlowDate + list1.get(a + 1).getFlowHour();
                        }
                        //写入数据
                        Map<String,Object> map = new HashMap<>();
                        map.put("startTime",startTime);
                        map.put("endTime",endTime);
                        map.put("weekNumber",weekNumber);
                        map.put("weekDay",day);
                        map.put("peakType",isMoning(startTime));
                        map.put("dayType",isWorkDate(day));
                        map.put("weekStartTime",todayOfWeek.toString());
                        map.put("weekEndTime",endOfWeek.toString());
                        map.put("maxFlow",maxFlow);
                        map.put("maxFlowDate",maxFlowDate);
                        map.put("sumFlowDate",sumFlowDate);
                        map.put("crossId",temp.getCrossId());
                        analysisGreenCongestionPeriodMapper.insertCrossDataHistByDay(map);
                        //清空数据
                        startTime = "";
                        endTime = "";
                        list.clear();
                        queueCorss.clear();
                    }else {
                        if(!StringUtil.isEmpty(startTime)) {
                            list.add(temp);
                        }
                    }
                }
                enqueue(queue,temp);
            }else {
                queue.clear();
                queueCorss.clear();
                list.clear();
                crossId = temp.getCrossId();
                //计算该路口交通指数阈值，取一天的9点至16点30之间交通指数倒序的90%的数值为动态阈值
                try {
                    List<laneDataHist> crossTrafficList = analysisGreenCongestionPeriodMapper.selectCrossTrafficThreshold(date.toString().replaceAll("-", ""), crossId);
                    if (!crossTrafficList.isEmpty() && crossTrafficList.get(9).getTrafficIndex() <= 1.5) {
                        trafficThreshold = 1.5;
                    } else {
                        trafficThreshold = crossTrafficList.get(9).getTrafficIndex();
                    }
                    startTime = "";
                    endTime = "";
                    enqueue(queue, temp);
                }catch (Exception e) {
                    log.error("",e);
                }
            }
        }
        queue.clear();
        queueCorss.clear();
        crossId = "";
        startTime = "";
        endTime = "";
        trafficThreshold = 0.0;
        //计算平峰
        List<laneDataHist> peakDetailList = analysisGreenCongestionPeriodMapper.selectCrossPeakDetail(date.toString());
        for(int b = 0 ; b < peakDetailList.size() ; b++){
            if(b == peakDetailList.size() - 1 ){
                if(queue.size() > 0){
                    if(queue.getFirst().getCrossId().equals(peakDetailList.get(b).getCrossId())){
                        if(!queue.getFirst().getPeakType().equals(peakDetailList.get(b).getPeakType())){
                            startTime = queue.getFirst().getEndTime();
                            endTime = peakDetailList.get(b).getStartTime();
                        }else {
                            startTime = queue.getFirst().getEndTime();
                            endTime = date.toString() + " 20:00:00";
                        }
                    }else {
                        startTime = queue.getFirst().getEndTime();
                        endTime = date.toString() + " 20:00:00";
                        List<laneDataHist> crossFlowList = analysisGreenCongestionPeriodMapper.selectCrossDataHistFlow(date.toString().replaceAll("-",""),
                                queue.getFirst().getCrossId());
                        List<laneDataHist> list1 = new ArrayList<>();
                        Integer maxFlow = 0;
                        Integer maxFlowDate = 0;
                        Integer sumFlowDate = 0;
                        if(CollectionUtils.isNotEmpty(crossFlowList)) {
                            for (laneDataHist temp1 : crossFlowList) {
                                if (queueCorss.size() < 2) {
                                    enqueue(queueCorss, temp1);
                                } else {
                                    Integer flow = (temp1.getFlow() + queueCorss.getFirst().getFlow() + queueCorss.getLast().getFlow()) * 4 ;
                                    temp1.setFlowHour(flow);
                                    list1.add(temp1);
                                    enqueue(queueCorss, temp1);
                                }
                            }

                            maxFlowDate = list1.get(0).getFlowHour();
                            sumFlowDate = list1.get(0).getFlowHour();
                        }

                        for(int a = 0 ; a < list1.size(); a ++){
                            if(a == list1.size() - 1){
                                continue;
                            }
                            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                            Date startTimeDate = format.parse(startTime);
                            Date endTimeDate = format.parse(endTime);
                            Date tempdate = format.parse(list1.get(a).getStartTime());
                            if(tempdate.compareTo(startTimeDate) >= 0 && tempdate.compareTo(endTimeDate) < 0 )
                            {
                                if(list1.get(a + 1).getFlowHour() > list1.get(a).getFlowHour()){
                                    maxFlow = list1.get(a + 1).getFlowHour();
                                }else {
                                    maxFlow = list1.get(a).getFlowHour();
                                }
                            }
                            if(list1.get(a + 1).getFlowHour() > maxFlowDate){
                                maxFlowDate = list1.get(a + 1).getFlowHour();
                            }
                            sumFlowDate = sumFlowDate + list1.get(a + 1).getFlowHour();
                        }
                        //写入数据
                        Map<String,Object> map = new HashMap<>();
                        map.put("startTime",startTime);
                        map.put("endTime",endTime);
                        map.put("weekNumber",weekNumber);
                        map.put("weekDay",day);
                        map.put("peakType",3);
                        map.put("dayType",isWorkDate(day));
                        map.put("weekStartTime",todayOfWeek.toString());
                        map.put("weekEndTime",endOfWeek.toString());
                        map.put("maxFlow",maxFlow);
                        map.put("maxFlowDate",maxFlowDate);
                        map.put("sumFlowDate",sumFlowDate);
                        map.put("crossId",queue.getFirst().getCrossId());
                        analysisGreenCongestionPeriodMapper.insertCrossDataHistByDay(map);
                        queueCorss.clear();
                        if(peakDetailList.get(b).getPeakType().equals("2")) {
                            startTime = date.toString() + " 08:00:00";
                            endTime = peakDetailList.get(b).getStartTime();

                        }else {
                            startTime = peakDetailList.get(b).getEndTime();
                            endTime = date.toString() + " 20:00:00";
                        }
                    }
                }else {
                    if(peakDetailList.get(b).getPeakType().equals("2")) {
                        startTime = date.toString() + " 08:00:00";
                        endTime = peakDetailList.get(b).getStartTime();

                    }else {
                        startTime = peakDetailList.get(b).getEndTime();
                        endTime = date.toString() + " 20:00:00";
                    }
                }
                List<laneDataHist> crossFlowList = analysisGreenCongestionPeriodMapper.selectCrossDataHistFlow(date.toString().replaceAll("-",""),
                        peakDetailList.get(b).getCrossId());
                List<laneDataHist> list1 = new ArrayList<>();
                Integer maxFlow = 0;
                Integer maxFlowDate = 0;
                Integer sumFlowDate = 0;
                if(CollectionUtils.isNotEmpty(crossFlowList)) {
                    for (laneDataHist temp1 : crossFlowList) {
                        if (queueCorss.size() < 2) {
                            enqueue(queueCorss, temp1);
                        } else {
                            Integer flow = (temp1.getFlow() + queueCorss.getFirst().getFlow() + queueCorss.getLast().getFlow()) * 4 ;
                            temp1.setFlowHour(flow);
                            list1.add(temp1);
                            enqueue(queueCorss, temp1);
                        }
                    }

                    maxFlowDate = list1.get(0).getFlowHour();
                    sumFlowDate = list1.get(0).getFlowHour();
                }

                for(int a = 0 ; a < list1.size(); a ++){
                    if(a == list1.size() - 1){
                        continue;
                    }
                    SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    Date startTimeDate = format.parse(startTime);
                    Date endTimeDate = format.parse(endTime);
                    Date tempdate = format.parse(list1.get(a).getStartTime());
                    if(tempdate.compareTo(startTimeDate) >= 0 && tempdate.compareTo(endTimeDate) < 0 )
                    {
                        if(list1.get(a + 1).getFlowHour() > list1.get(a).getFlowHour()){
                            maxFlow = list1.get(a + 1).getFlowHour();
                        }else {
                            maxFlow = list1.get(a).getFlowHour();
                        }
                    }
                    if(list1.get(a + 1).getFlowHour() > maxFlowDate){
                        maxFlowDate = list1.get(a + 1).getFlowHour();
                    }
                    sumFlowDate = sumFlowDate + list1.get(a + 1).getFlowHour();
                }
                //写入数据
                Map<String,Object> map = new HashMap<>();
                map.put("startTime",startTime);
                map.put("endTime",endTime);
                map.put("weekNumber",weekNumber);
                map.put("weekDay",day);
                map.put("peakType",3);
                map.put("dayType",isWorkDate(day));
                map.put("weekStartTime",todayOfWeek.toString());
                map.put("weekEndTime",endOfWeek.toString());
                map.put("maxFlow",maxFlow);
                map.put("maxFlowDate",maxFlowDate);
                map.put("sumFlowDate",sumFlowDate);
                map.put("crossId",peakDetailList.get(b).getCrossId());
                analysisGreenCongestionPeriodMapper.insertCrossDataHistByDay(map);
                startTime = "";
                endTime = "";
                queue.clear();
                queueCorss.clear();
                continue;

            }
            if(peakDetailList.get(b).getCrossId().equals(crossId)){
                if(queue.size() > 0) {
                    if (peakDetailList.get(b).getPeakType().equals("2") && queue.getFirst().getPeakType().equals("1")) {
                        startTime = queue.getFirst().getEndTime();
                        endTime = peakDetailList.get(b).getStartTime();
                        List<laneDataHist> crossFlowList = analysisGreenCongestionPeriodMapper.selectCrossDataHistFlow(date.toString().replaceAll("-",""),
                                peakDetailList.get(b).getCrossId());
                        List<laneDataHist> list1 = new ArrayList<>();
                        Integer maxFlow = 0;
                        Integer maxFlowDate = 0;
                        Integer sumFlowDate = 0;
                        if(CollectionUtils.isNotEmpty(crossFlowList)) {
                            for (laneDataHist temp1 : crossFlowList) {
                                if (queueCorss.size() < 2) {
                                    enqueue(queueCorss, temp1);
                                } else {
                                    Integer flow = (temp1.getFlow() + queueCorss.getFirst().getFlow() + queueCorss.getLast().getFlow()) * 4 ;
                                    temp1.setFlowHour(flow);
                                    list1.add(temp1);
                                    enqueue(queueCorss, temp1);
                                }
                            }

                            maxFlowDate = list1.get(0).getFlowHour();
                            sumFlowDate = list1.get(0).getFlowHour();
                        }

                        for(int a = 0 ; a < list1.size(); a ++){
                            if(a == list1.size() - 1){
                                continue;
                            }
                            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                            Date startTimeDate = format.parse(startTime);
                            Date endTimeDate = format.parse(endTime);
                            Date tempdate = format.parse(list1.get(a).getStartTime());
                            if(tempdate.compareTo(startTimeDate) >= 0 && tempdate.compareTo(endTimeDate) < 0 )
                            {
                                if(list1.get(a + 1).getFlowHour() > list1.get(a).getFlowHour()){
                                    maxFlow = list1.get(a + 1).getFlowHour();
                                }else {
                                    maxFlow = list1.get(a).getFlowHour();
                                }
                            }
                            if(list1.get(a + 1).getFlowHour() > maxFlowDate){
                                maxFlowDate = list1.get(a + 1).getFlowHour();
                            }
                            sumFlowDate = sumFlowDate + list1.get(a + 1).getFlowHour();
                        }
                        //写入数据
                        Map<String,Object> map = new HashMap<>();
                        map.put("startTime",startTime);
                        map.put("endTime",endTime);
                        map.put("weekNumber",weekNumber);
                        map.put("weekDay",day);
                        map.put("peakType",3);
                        map.put("dayType",isWorkDate(day));
                        map.put("weekStartTime",todayOfWeek.toString());
                        map.put("weekEndTime",endOfWeek.toString());
                        map.put("maxFlow",maxFlow);
                        map.put("maxFlowDate",maxFlowDate);
                        map.put("sumFlowDate",sumFlowDate);
                        map.put("crossId",peakDetailList.get(b).getCrossId());
                        analysisGreenCongestionPeriodMapper.insertCrossDataHistByDay(map);
                        startTime = "";
                        endTime = "";
                        queue.clear();
                        queueCorss.clear();
                    }
                }
            }else {
                if(queue.size() == 1){
                    if(queue.getFirst().getPeakType().equals("1")) {
                        startTime = queue.getFirst().getEndTime();
                        endTime = date.toString() + " 20:00:00";
                    }else {
                        startTime = date.toString() + " 08:00:00";
                        endTime = queue.getFirst().getStartTime();
                    }
                    List<laneDataHist> crossFlowList = analysisGreenCongestionPeriodMapper.selectCrossDataHistFlow(date.toString().replaceAll("-",""),
                            queue.getFirst().getCrossId());
                    List<laneDataHist> list1 = new ArrayList<>();
                    Integer maxFlow = 0;
                    Integer maxFlowDate = 0;
                    Integer sumFlowDate = 0;
                    if(CollectionUtils.isNotEmpty(crossFlowList)) {
                        for (laneDataHist temp1 : crossFlowList) {
                            if (queueCorss.size() < 2) {
                                enqueue(queueCorss, temp1);
                            } else {
                                Integer flow = (temp1.getFlow() + queueCorss.getFirst().getFlow() + queueCorss.getLast().getFlow()) * 4 ;
                                temp1.setFlowHour(flow);
                                list1.add(temp1);
                                enqueue(queueCorss, temp1);
                            }
                        }

                        maxFlowDate = list1.get(0).getFlowHour();
                        sumFlowDate = list1.get(0).getFlowHour();
                    }

                    for(int a = 0 ; a < list1.size(); a ++){
                        if(a == list1.size() - 1){
                            continue;
                        }
                        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                        Date startTimeDate = format.parse(startTime);
                        Date endTimeDate = format.parse(endTime);
                        Date tempdate = format.parse(list1.get(a).getStartTime());
                        if(tempdate.compareTo(startTimeDate) >= 0 && tempdate.compareTo(endTimeDate) < 0 )
                        {
                            if(list1.get(a + 1).getFlowHour() > list1.get(a).getFlowHour()){
                                maxFlow = list1.get(a + 1).getFlowHour();
                            }else {
                                maxFlow = list1.get(a).getFlowHour();
                            }
                        }
                        if(list1.get(a + 1).getFlowHour() > maxFlowDate){
                            maxFlowDate = list1.get(a + 1).getFlowHour();
                        }
                        sumFlowDate = sumFlowDate + list1.get(a + 1).getFlowHour();
                    }
                    //写入数据
                    Map<String,Object> map = new HashMap<>();
                    map.put("startTime",startTime);
                    map.put("endTime",endTime);
                    map.put("weekNumber",weekNumber);
                    map.put("weekDay",day);
                    map.put("peakType",3);
                    map.put("dayType",isWorkDate(day));
                    map.put("weekStartTime",todayOfWeek.toString());
                    map.put("weekEndTime",endOfWeek.toString());
                    map.put("maxFlow",maxFlow);
                    map.put("maxFlowDate",maxFlowDate);
                    map.put("sumFlowDate",sumFlowDate);
                    map.put("crossId",queue.getFirst().getCrossId());
                    analysisGreenCongestionPeriodMapper.insertCrossDataHistByDay(map);
                    startTime = "";
                    endTime = "";
                    queue.clear();
                    queueCorss.clear();
                }
                queue.clear();
                crossId = peakDetailList.get(b).getCrossId();
                enqueue(queue,peakDetailList.get(b));
            }
        }
    }
}
