package net.wanji.opt.task;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.servicev2.evaluation.EventInfoService;
import net.wanji.opt.servicev2.judgeanalysis.AnalysisGreenCongestionPeriodService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Configurable;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.text.ParseException;

@Component
@Configurable
@EnableScheduling
@Slf4j
@Profile("!dev")
public class AnalysisGreenCongestionPeriodWeekTask {
    @Autowired
    private AnalysisGreenCongestionPeriodService analysisGreenCongestionPeriodService;

    @Resource
    EventInfoService eventInfoService;

    @Scheduled(cron = "0 30 2 ? * 1")
    //@PostConstruct
    public void task() throws ParseException {

        try {
            long st = System.currentTimeMillis();
            analysisGreenCongestionPeriodService.selectCountByCongestionPeriod();
            long et = System.currentTimeMillis();
            log.info("计算任务 selectCountByCongestionPeriod 耗时：{}ms", et - st);
        } catch (Exception e) {
            log.error("", e);
        }

        try {
            long st = System.currentTimeMillis();
            analysisGreenCongestionPeriodService.selectCountByLandData();
            long et = System.currentTimeMillis();
            log.info("计算任务 selectCountByLandData 耗时：{}ms", et - st);
        } catch (Exception e) {
            log.error("", e);
        }
    }

    @Scheduled(cron = "0 0 2 ? * ? ")
    public void taskDay() throws ParseException {

        try {
            long st = System.currentTimeMillis();
            analysisGreenCongestionPeriodService.selectCountByCrossDataOneDay();
            long et = System.currentTimeMillis();
            log.info("计算任务 taskDay 耗时：{}ms", et - st);
        } catch (Exception e) {
            log.error("", e);
        }
    }

    @Scheduled(cron = "0 0 2 ? * ? ")
    public void insertCrossSlowRunEventData() throws ParseException {
        long st = System.currentTimeMillis();
        try {

            DateTime endTDateTime = DateUtil.date();
            //开始时间
            DateTime startTDateTime = DateUtil.offsetDay(endTDateTime, -1);
            startTDateTime = DateUtil.beginOfDay(startTDateTime);
            String sdt = startTDateTime.toString("yyyy-MM-dd HH:00:00");

            //窗口截止时间
            endTDateTime = DateUtil.parseDateTime(sdt).offset(DateField.HOUR, 24).offset(DateField.SECOND, -1);
            String edt = endTDateTime.toString("yyyy-MM-dd HH:mm:ss");

            eventInfoService.insertCrossSlowRunEventData(sdt, edt);

        } catch (Exception e) {
            log.error("", e);
        }
        long et = System.currentTimeMillis();
        log.info("计算任务 insertCrossSlowRunEventData 耗时：{}ms", et - st);
    }
}
