package net.wanji.opt.task;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.servicev2.judgeanalysis.AnalysisProblemCrossDayService;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.Date;

@Component
@Slf4j
public class AnalysisProblemCrossDayTask implements ApplicationRunner {
    @Resource
    private AnalysisProblemCrossDayService analysisProblemCrossDayService;

    @Override
    public void run(ApplicationArguments args) throws Exception {
//        log.info("服务启动时--干线协调方向频发拥堵/缓行路段--任务执行时间：" + LocalDateTime.now());
        // 干线协调方向频发拥堵/缓行路段
        try {
//            analysisProblemCrossDayService.insertAnalysisProblemCrossDay();
        } catch (Exception e) {
            log.error("服务启动时--干线协调方向频发拥堵/缓行路段--任务执行错误" + e);
        }
//        log.info("服务启动时--干线协调方向频发拥堵/缓行路段--任务执行结束时间：" + LocalDateTime.now());
    }

    /**
     * 每凌晨 0:30 执行的任务
     */
    @Scheduled(cron = "0 10 0 * * ?")
    public void executeWeeklyTask() {
        //干线协调方向频发拥堵/缓行路段
        log.info("定时任务--干线协调方向频发拥堵/缓行路段--执行开始时间：" + LocalDateTime.now());
        try {
            analysisProblemCrossDayService.insertAnalysisProblemCrossDay();
        } catch (Exception e) {
            log.error("定时任务--干线协调方向频发拥堵/缓行路段--任务执行错误" + e);
        }
        log.info("定时任务--干线协调方向频发拥堵/缓行路段--执行结束时间：" + LocalDateTime.now());
    }

    /**
     * 每小时执行溢出
     */
    @Scheduled(cron = "5 0 * * * ?")
    public void executeGreenCrossHourTask() {
        //干线协调方向频发拥堵/缓行路段
        log.info("定时任务--干线协调方向频发拥堵/缓行路段--执行开始时间：" + LocalDateTime.now());
        try {
            DateTime endTDateTime = DateUtil.date();

            //开始时间
            DateTime startTDateTime = DateUtil.offsetHour(endTDateTime, -1);
            String sdt = startTDateTime.toString("yyyy-MM-dd HH:00:00");

            //窗口截止时间
            endTDateTime =DateUtil.parseDateTime(sdt).offset(DateField.HOUR,1).offset(DateField.SECOND,-1);
            String edt = endTDateTime.toString("yyyy-MM-dd HH:mm:ss");

            analysisProblemCrossDayService.insertGreenCrossHourData(sdt, edt);

            log.info("干线协调方向频发拥堵/缓行路段=》小时粒度数据入库，小时开始时间：{},小时截止时间：{}，截止时间：{}", sdt, edt, endTDateTime.toString("yyyy-MM-dd HH:mm:ss"));

        } catch (Exception e) {
            log.error("定时任务--干线协调方向频发拥堵/缓行路段--任务执行错误" + e);
        }

        log.info("定时任务--干线协调方向频发拥堵/缓行路段--执行结束时间：" + LocalDateTime.now());
    }


    public static void main(String[] args) {

        DateTime endTDateTime = DateUtil.date();

        //开始时间
        DateTime startTDateTime = DateUtil.offsetHour(endTDateTime, -1);
        String sdt = startTDateTime.toString("yyyy-MM-dd HH:00:00");

        //窗口截止时间
        DateTime endDateTime = DateUtil.parseDateTime(sdt).offset(DateField.HOUR,1).offset(DateField.SECOND,-1);
        String edt = endDateTime.toString("yyyy-MM-dd HH:mm:ss");

        System.out.println(sdt+" "+edt);
    }

}
