package net.wanji.opt.task;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import io.lettuce.core.dynamic.annotation.Param;
import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.dao.mapper.TablePartitionManagerMapper;
import org.springframework.beans.factory.annotation.Configurable;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;

@Component
@Configurable
@EnableScheduling
@Slf4j
@Profile("!dev")
public class TablePartitionManagerTask {
    @Resource
    TablePartitionManagerMapper tablePartitionManagerMapper;

    @Value("${partition.table.list}")
    private String partitionTableList;

    @Scheduled(cron = "0 1/60 * * * *")
    public  void createTablePartition() {
        String[] tableList = partitionTableList.split(",");

        DateTime tomorrow = DateTime.now().offset(DateField.HOUR,1);

        Integer dt =   Integer.parseInt(DateUtil.format(tomorrow.toJdkDate(), "yyyyMMdd"));

        for (String name : tableList) {
            try {
                String partitionName = name+"_"+dt;
                tablePartitionManagerMapper.createTablePartition(name, dt,partitionName);
                log.info("创建表分区，tableName：{},日期：{}，分区名称：{}",name,dt,partitionName);
            } catch (Exception e) {
                log.error("创建表分区失败", e);
            }

        }
    }


    public  void deleteTablePartitionData(){

        String[] tableList = partitionTableList.split(",");

        DateTime tomorrow = DateTime.now() ;

        Integer dt = 20250316;//  Integer.parseInt(DateUtil.format(tomorrow.toJdkDate(), "yyyyMMdd"));

        for (String name : tableList) {
            try {
                String partitionName = name+"_"+dt;
                tablePartitionManagerMapper.deleteTablePartitionData(partitionName);
                log.info("删除分区数据分区数据，tableName：{},日期：{}，分区名称：{}",name,dt,partitionName);
            } catch (Exception e) {
                log.error("删除分区数据分区失败", e);
            }
        }
    }

}
